/*
* BearLibTerminal
* Copyright (C) 2013 Cfyz
*
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
* of the Software, and to permit persons to whom the Software is furnished to do
* so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
* FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
* COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
* IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
* CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/

#ifndef BEARLIBTERMINAL_H
#define BEARLIBTERMINAL_H

#include <stddef.h>
#include <stdint.h>

// Keyboard scancodes
#define VK_LBUTTON			0x01
#define VK_RBUTTON			0x02
#define VK_CLOSE			0x03 // Same code as VK_CANCEL from winuser.h
#define VK_BACK				0x08 // Backspace
#define VK_BACKSPACE		0x08 // Backspace (alias)
#define VK_TAB				0x09 
#define VK_RETURN			0x0D // Enter
#define VK_SHIFT			0x10
#define VK_CONTROL			0x11
#define VK_PAUSE			0x13 // Pause/Break
#define VK_ESCAPE			0x1B
#define VK_SPACE			0x20
#define VK_PRIOR			0x21 // Page Up
#define VK_NEXT				0x22 // Page Down
#define VK_END				0x23
#define VK_HOME				0x24
#define VK_LEFT				0x25 // Left arrow
#define VK_UP				0x26 // Up arrow
#define VK_RIGHT			0x27 // Right arrow
#define VK_DOWN				0x28 // Down arrow
#define VK_INSERT			0x2D
#define VK_DELETE			0x2E
#define VK_0				0x30
#define VK_1				0x31
#define VK_2				0x32
#define VK_3				0x33
#define VK_4				0x34
#define VK_5				0x35
#define VK_6				0x36
#define VK_7				0x37
#define VK_8				0x38
#define VK_9				0x39
#define VK_A				0x41
#define VK_B				0x42
#define VK_C				0x43
#define VK_D				0x44
#define VK_E				0x45
#define VK_F				0x46
#define VK_G				0x47
#define VK_H				0x48
#define VK_I				0x49
#define VK_J				0x4A
#define VK_K				0x4B
#define VK_L				0x4C
#define VK_M				0x4D
#define VK_N				0x4E
#define VK_O				0x4F
#define VK_P				0x50
#define VK_Q				0x51
#define VK_R				0x52
#define VK_S				0x53
#define VK_T				0x54
#define VK_U				0x55
#define VK_V				0x56
#define VK_W				0x57
#define VK_X				0x58
#define VK_Y				0x59
#define VK_Z				0x5A
#define VK_GRAVE			0xC0 //  ` 
#define VK_MINUS			0xBD //  -
#define VK_EQUALS			0xBB //  =
#define VK_BACKSLASH		0xDC //  \ .
#define VK_LBRACKET			0xDB //  [
#define VK_RBRACKET			0xDD //  ]
#define VK_SEMICOLON		0xBA //  ;
#define VK_APOSTROPHE		0xDE //  '
#define VK_COMMA			0xBC //  ,
#define VK_PERIOD			0xBE //  .
#define VK_SLASH			0xBF //  /
#define VK_NUMPAD0			0x60
#define VK_NUMPAD1			0x61
#define VK_NUMPAD2			0x62
#define VK_NUMPAD3			0x63
#define VK_NUMPAD4			0x64
#define VK_NUMPAD5			0x65
#define VK_NUMPAD6			0x66
#define VK_NUMPAD7			0x67
#define VK_NUMPAD8			0x68
#define VK_NUMPAD9			0x69
#define VK_MULTIPLY			0x6A // '*' on numpad
#define VK_ADD				0x6B // '+' on numpad
#define VK_SUBTRACT			0x6D // '-' on numpad
#define VK_DECIMAL			0x6E // '.' on numpad
#define VK_DIVIDE			0x6F // '/' on numpad
#define VK_F1				0x70
#define VK_F2				0x71
#define VK_F3				0x72
#define VK_F4				0x73
#define VK_F5				0x74
#define VK_F6				0x75
#define VK_F7				0x76
#define VK_F8				0x77
#define VK_F9				0x78
#define VK_F10				0x79
#define VK_F11				0x7A
#define VK_F12				0x7B
#define VK_MOUSE_MOVE		0xD3 // Mouse movement event
#define VK_MOUSE_SCROLL		0xD4 // Mouse wheel scroll event
#define VK_FLAG_RELEASED	0x100 // This flag is set if key was released instead of pressed

// Virtual codes for various virtual states:
#define VK_MOUSE_X			0xD0 // Mouse cursor position in characters
#define VK_MOUSE_Y			0xD1
#define VK_MOUSE_WHEEL		0xD2
#define VK_MOUSE_PIXEL_X	0xD5 // Mouse cursor position in pixels
#define VK_MOUSE_PIXEL_Y	0xD6
#define VK_CELL_WIDTH		0xD7 // Character cell size in pixels
#define VK_CELL_HEIGHT		0xD8

// Color is in little-endian BGRA order, one byte per channel: 0xAARRGGBB
typedef uint32_t color_t;

// Some most basic colors (B/W + rainbow)
#define COLOR_WHITE			0xFFFFFFFF
#define COLOR_BLACK			0xFF000000
#define COLOR_RED			0xFFFF0000
#define COLOR_ORANGE		0xFFFF8000
#define COLOR_YELLOW		0xFFFFFF00
#define COLOR_GREEN			0xFF00FF00
#define COLOR_CYAN			0xFF00FFFF
#define COLOR_BLUE			0xFF0000FF
#define COLOR_VIOLET		0xFFFF00FF

// Some codepage symbolic constants for terminal_open API call
// NOTE: ANSI/OEM one-byte codepages go by their codes e.g. just 1251 for cyrillic windows-1251
// NOTE: strictly speaking UTF is an encoding but the term codepage is used for better consistency
#define CODEPAGE_UTF8		65001
#define CODEPAGE_DEFAULT	CODEPAGE_UTF8

// 'Blending' modes
#define BLENDING_NONE		0
#define BLENDING_ADDITIVE	1

typedef void (*custom_render_cb_t)();

#ifdef _WIN32
#  ifdef BEARLIBTERMINAL_LIBRARY
#    define TERMINAL_API __declspec(dllexport)
#  else
#    define TERMINAL_API __declspec(dllimport)
#  endif
#else
#  define TERMINAL_API
#endif

#ifdef __cplusplus
extern "C" {
#endif

TERMINAL_API int terminal_open(int codepage);
TERMINAL_API void terminal_close();
TERMINAL_API int terminal_options(const char* options, ...);
TERMINAL_API int terminal_options_noformat(const char* options);
TERMINAL_API void terminal_custom_render(custom_render_cb_t callback);
TERMINAL_API void terminal_refresh();
TERMINAL_API void terminal_clear();
TERMINAL_API void terminal_clear_area(int x, int y, int w, int h);
TERMINAL_API void terminal_color(color_t color);
TERMINAL_API void terminal_bkcolor(color_t color);
TERMINAL_API void terminal_blending(int mode);
TERMINAL_API void terminal_put(int x, int y, int code);
TERMINAL_API size_t terminal_printf(int x, int y, const char* format, ...);
TERMINAL_API size_t terminal_printf_noformat(int x, int y, const char* s);
TERMINAL_API int terminal_has_input();
TERMINAL_API int terminal_state(int code);
TERMINAL_API int terminal_get();
TERMINAL_API color_t color_from_name(const char* name);
TERMINAL_API int terminal_use_image(int code, const char* filename);

// NYI
TERMINAL_API void terminal_custom_render(custom_render_cb_t callback);
TERMINAL_API int terminal_getstr(int x, int y, char* buffer, int max);

#undef TERMINAL_API

// Utility funcitions

inline color_t color_from_argb(int a, int r, int g, int b)
{
	if (a > 255) a = 255;
	if (r > 255) r = 255;
	if (g > 255) g = 255;
	if (b > 255) b = 255;
	return (a << 24) | (r << 16) | (g << 8) | b;
}

inline int color_alpha(color_t color) {return (color & 0xFF000000) >> 24;}

inline int color_red(color_t color) {return (color & 0x00FF0000) >> 16;}

inline int color_green(color_t color) {return (color & 0x0000FF00) >> 8;}

inline int color_blue(color_t color) {return (color & 0x000000FF);}

#ifdef __cplusplus
} // End of extern "C"

#include <string>
#include <vector>

inline color_t color_from_name(const std::string& name)
{
	return color_from_name(name.c_str());
}

/*
inline int terminal_getstr(int x, int y, std::string& s, int max, int timeout)
{
	s.reserve(max*3+1);
	return terminal_getstr(x, y, &s[0], max);
}
//*/

#endif // __cplusplus

#endif /* BEARLIBTERMINAL_H */
