/*
* BearLibTerminal
* Copyright (C) 2013 Cfyz
*
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
* of the Software, and to permit persons to whom the Software is furnished to do
* so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
* FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
* COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
* IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
* CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/

#ifndef BEARLIBTERMINAL_ENCODING_HPP
#define BEARLIBTERMINAL_ENCODING_HPP

#include <string>
#include <vector>
#include <map>
#include <cstdint>

namespace BearLibTerminal
{
	class Encoding
	{
	public:
		Encoding();
		Encoding(int codepage);
		Encoding(const std::wstring& name);
		std::wstring ToUCS2(const std::string& s);
		wchar_t ToUCS2(int code);
		std::string FromUCS2(const std::wstring& s);
		int FromUCS2(wchar_t code) const; // TODO: const everywhere

		static Encoding UTF8;

	private:
		void FillMaps(const wchar_t* table);
		int m_codepage;
		std::vector<wchar_t> m_forward_map;
		std::map<wchar_t, std::uint16_t> m_backward_map;
	};
}

#endif // BEARLIBTERMINAL_ENCODING_HPP
