/*
* BearLibTerminal
* Copyright (C) 2013 Cfyz
*
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
* of the Software, and to permit persons to whom the Software is furnished to do
* so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
* FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
* COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
* IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
* CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/

#ifndef BEARLIBTERMINAL_FONTPROVIDER_HPP
#define BEARLIBTERMINAL_FONTPROVIDER_HPP

#include <string>
#include <list>
#include "Size.hpp"
#include "Bitmap.hpp"
#include "GlyphTileset.hpp"

#include <map>
#include <string>
#include <cstdint>

namespace BearLibTerminal
{
	class FontProvider
	{
	public:
		struct Options
		{
			Options();
			bool operator== (const Options& other) const;
			bool operator!= (const Options& other) const;
			std::wstring name;
			Size size;
			std::wstring codepage;
			std::wstring mode;
			std::wstring supplement;
		};

		FontProvider(const Options& options);
		virtual ~FontProvider() { }
		virtual void Dispose() = 0;
		virtual Bitmap GetGlyphBitmap(wchar_t code);
		virtual bool HasGlyph(wchar_t code) const;
		virtual void PopulateTileset(GlyphTileset& tileset);
		virtual void PopulateTileset(GlyphTileset& tileset, const std::list<std::uint16_t>& codes);
		inline Size GetGlyphSize() const { return m_glyph_size; }
		inline Size GetCellSize() const { return m_cell_size; }

	protected:
		Size m_glyph_size;
		Size m_cell_size;
		Bitmap m_supplement_cache;
	};
}

#endif // BEARLIBTERMINAL_FONTPROVIDER_HPP
