/*
* BearLibTerminal
* Copyright (C) 2013 Cfyz
*
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
* of the Software, and to permit persons to whom the Software is furnished to do
* so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
* FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
* COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
* IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
* CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/

#include <stdexcept>
#include "FontProviderFactory.hpp"
#include "Utility.hpp"
#include "BitmapFontProvider.hpp"
#include "FreetypeFontProvider.hpp"
#include "DefaultFontProvider.hpp"

namespace BearLibTerminal
{
	std::shared_ptr<FontProvider> FontProviderFactory::Create(const FontProvider::Options& options)
	{
		// Guess font type from name

		if ( options.name.empty() || options.name == L"default" )
		{
			return std::make_shared<DefaultFontProvider>(options);
		}
		else if ( ends_with<wchar_t>(tolower(options.name), L".ttf") )
		{
			return std::make_shared<FreetypeFontProvider>(options);
		}
		else if ( ends_with<wchar_t>(tolower(options.name), L".png") || ends_with<wchar_t>(tolower(options.name), L".bmp") )
		{
			return std::make_shared<BitmapFontProvider>(options);
		}
		else
		{
			throw std::runtime_error("font name is not valid");
		}
	}
}
