/*
* BearLibTerminal
* Copyright (C) 2013 Cfyz
*
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
* of the Software, and to permit persons to whom the Software is furnished to do
* so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
* FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
* COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
* IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
* CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/

#ifndef BEARLIBTERMINAL_GEOMETRY_HPP
#define BEARLIBTERMINAL_GEOMETRY_HPP

#include <ostream>
#include <istream>
#include "Point.hpp"
#include "Size.hpp"
#include "Rectangle.hpp"

namespace BearLibTerminal
{
	// Point + Size
	template<typename T> BasicPoint<T> operator+ (const BasicPoint<T>& left, const BasicSize<T>& right)
	{
		return BasicPoint<T>(left.x+right.width, left.y+right.height);
	}

	// Point - Size
	template<typename T> BasicPoint<T> operator- (const BasicPoint<T>& left, const BasicSize<T>& right)
	{
		return BasicPoint<T>(left.x-right.width, left.y-right.height);
	}

	// Point * Size
	template<typename T> BasicPoint<T> operator* (const BasicPoint<T>& left, const BasicSize<T>& right)
	{
		return BasicPoint<T>(left.x*right.width, left.y*right.height);
	}

	// Size * Point
	template<typename T> BasicPoint<T> operator* (const BasicSize<T>& left, const BasicPoint<T>& right)
	{
		return right*left;
	}

	// Rectangle * Size
	template<typename T> BasicRectangle<T> operator* (const BasicRectangle<T>& left, const BasicSize<T>& right)
	{
		return BasicRectangle<T>(left.left*right.width, left.top*right.height, left.width*right.width, left.height*right.height);
	}

	// Point / Size
	template<typename T> BasicPoint<T> operator/ (const BasicPoint<T>& left, const BasicSize<T>& right)
	{
		return BasicPoint<T>(left.x / right.width, left.y / right.height);
	}

	// Rectangle / Size
	template<typename T> BasicRectangle<T> operator/ (const BasicRectangle<T>& left, const BasicSize<T>& right)
	{
		return BasicRectangle<T>(left.left/right.width, left.top/right.height, left.width/right.width, left.height/right.height);
	}

	// stream << Size
	template<typename char_t, typename T> std::basic_ostream<char_t>& operator<< (std::basic_ostream<char_t>& stream, const BasicSize<T>& s)
	{
			stream << s.width << "x" << s.height;
			return stream;
	}

	// stream >> Size
	template<typename char_t, typename T> std::basic_istream<char_t>& operator>> (std::basic_istream<char_t>& stream, BasicSize<T>& s)
	{
			stream >> s.width;
			stream.ignore(1);
			stream >> s.height;
			return stream;
	}

	// Size<T2> -> Size<T1>
	template<typename T1, typename T2> BasicSize<T1> As(const BasicSize<T2>& from)
	{
		return BasicSize<T1>(T1(from.width), T1(from.height));
	}
}

#endif // BEARLIBTERMINAL_GEOMETRY_HPP
