/*
* BearLibTerminal
* Copyright (C) 2013 Cfyz
*
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
* of the Software, and to permit persons to whom the Software is furnished to do
* so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
* FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
* COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
* IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
* CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/

#include "GlyphTileset.hpp"

namespace BearLibTerminal
{
	GlyphTileset::GlyphTileset():
		m_glyphs(0x10000, 0xFFFF)
	{ }

	size_t GlyphTileset::GetGlyphCount() const
	{
		return m_tileset.GetTileCount();
	}

	Size GlyphTileset::GetGlyphSize() const
	{
		return m_tileset.GetTileSize();
	}

	void GlyphTileset::Clear()
	{
		m_tileset.Clear();
		m_glyphs.assign(0x10000, 0xFFFF);
	}

	void GlyphTileset::Dispose()
	{
		m_tileset.Dispose();
		m_glyphs.assign(0x10000, 0xFFFF);
	}

	bool GlyphTileset::HasTileFor(uint16_t code) const
	{
		return m_glyphs[code] != 0xFFFF;
	}

	void GlyphTileset::Bind()
	{
		m_tileset.Bind();
	}

	void GlyphTileset::Add(uint16_t code, const Bitmap& bitmap)
	{
		size_t index = m_tileset.Add(bitmap);
		m_glyphs[code] = index;
	}

	void GlyphTileset::Add(uint16_t code, const Bitmap& bitmap, const Rectangle& area)
	{
		size_t index = m_tileset.Add(bitmap, area);
		m_glyphs[code] = index;
	}

	void GlyphTileset::Add(uint16_t code, uint16_t alias)
	{
		m_glyphs[code] = m_glyphs[alias];
	}

	void GlyphTileset::Draw(uint16_t code, Point location) const
	{
		uint16_t index = m_glyphs[code];
		m_tileset.Draw(index, location);
	}
}
