/*
* BearLibTerminal
* Copyright (C) 2013 Cfyz
*
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
* of the Software, and to permit persons to whom the Software is furnished to do
* so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
* FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
* COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
* IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
* CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/

#ifndef BEARLIBTERMINAL_LOG_HPP
#define BEARLIBTERMINAL_LOG_HPP

#include <string>
#include <sstream>
#include <mutex>
#include <cstdint>
#include <iostream>

namespace BearLibTerminal
{
	class Log
	{
	public:
		enum class Level {None, Fatal, Error, Warning, Info, Debug, Trace};

		struct Options
		{
			Options();
			Options(const std::wstring& file, Level level);
			std::wstring file;
			Level level;
		};

		static void Write(Level level, const std::wstring& what);
		static void SetOptions(const Options& options);
		static Level GetLevel();

	private:
		static std::mutex m_lock;
		static Options m_options;
	};

	std::wostream& operator<< (std::wostream& stream, const Log::Level& value);
	std::wistream& operator>> (std::wistream& stream, Log::Level& value);
}

#define LOG(level, what)\
	do {\
		if (Log::Level::level <= BearLibTerminal::Log::GetLevel()) {\
			std::wostringstream stream;\
			stream << what;\
			BearLibTerminal::Log::Write(Log::Level::level, stream.str());\
		}\
	} while ( 0 )

#ifdef _MSC_VER
std::wstring GetLastErrorStr(std::uint32_t rc);
std::wstring GetLastErrorStr();
#endif

#endif // BEARLIBTERMINAL_LOG_HPP
