/*
* BearLibTerminal
* Copyright (C) 2013 Cfyz
*
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
* of the Software, and to permit persons to whom the Software is furnished to do
* so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
* FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
* COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
* IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
* CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/

#ifndef BEARLIBTERMINAL_OPTIONS_HPP
#define BEARLIBTERMINAL_OPTIONS_HPP

#include <cstdint>
#include <string>
#include "BearLibTerminal.h"
#include "Keystroke.hpp"
#include "Log.hpp"
#include "FontProvider.hpp"

namespace BearLibTerminal
{
	struct Timeout
	{
		Timeout();
		Timeout(int value);
		operator int() const;
		int value;

		static Timeout Indefinite, Immediate;
	};

	std::wostream& operator<<(std::wostream& stream, const Timeout& value);
	std::wistream& operator>>(std::wistream& stream, Timeout& value);

	struct InputFilter
	{
		InputFilter();
		InputFilter(unsigned int value);
		operator unsigned int() const;
		bool Has(unsigned int value) const;
		unsigned int value;

		static InputFilter None, KeyPress, KeyRelease, MouseMove, MouseScroll, All;
	};

	std::wostream& operator<<(std::wostream& stream, const InputFilter& value);
	std::wistream& operator>>(std::wistream& stream, InputFilter& value);

	struct Options
	{
		Options();
		Size size;
		std::wstring title;
		std::wstring icon;
		bool output_postformat;
		bool output_synchronous;
		custom_render_cb_t output_custom_render;
		Log::Options log;
		FontProvider::Options font;
		InputFilter input_events;
		bool input_precise_mouse;
		bool input_sticky_close;
		Timeout input_timeout;
	};
}

#endif // BEARLIBTERMINAL_OPTIONS_HPP
