/*
* BearLibTerminal
* Copyright (C) 2013 Cfyz
*
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
* of the Software, and to permit persons to whom the Software is furnished to do
* so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
* FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
* COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
* IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
* CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/

#ifndef BEARLIBTERMINAL_RECTANGLE_HPP
#define BEARLIBTERMINAL_RECTANGLE_HPP

#include "Point.hpp"
#include "Size.hpp"

namespace BearLibTerminal
{
	template<typename T> struct BasicRectangle
	{
		typedef T value_type;

		BasicRectangle():
			left(),
			top(),
			width(),
			height()
		{ }

		BasicRectangle(T left, T top, T width, T height):
			left(left),
			top(top),
			width(width),
			height(height)
		{ }

		BasicRectangle(BasicSize<T> size):
			left(),
			top(),
			width(size.width),
			height(size.height)
		{ }

		BasicRectangle(BasicPoint<T> location, BasicSize<T> size):
			left(location.x),
			top(location.y),
			width(size.width),
			height(size.height)
		{ }

		inline BasicPoint<T> Location() const
		{
			return BasicPoint<T>(left, top);
		}

		inline BasicSize<T> Size() const
		{
			return BasicSize<T>(width, height);
		}

		inline T Area() const
		{
			return width*height;
		}

		inline bool Contains(const BasicPoint<T>& point) const
		{
			return
				(point.x >= left) &&
				(point.y >= top) &&
				(point.x < left+width) &&
				(point.y < top+height);
		}

		inline bool Contains(const BasicRectangle<T>& rectangle) const
		{
			return
				(rectangle.left >= left) &&
				(rectangle.top >= top) &&
				(rectangle.left+rectangle.width <= left+width) &&
				(rectangle.top+rectangle.height <= top+height);
		}

		T left, top, width, height;
	};

	typedef BasicRectangle<int> Rectangle;

	typedef BasicRectangle<float> RectangleF;
}

#endif // BEARLIBTERMINAL_RECTANGLE_HPP
