/*
* BearLibTerminal
* Copyright (C) 2013 Cfyz
*
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
* of the Software, and to permit persons to whom the Software is furnished to do
* so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
* FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
* COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
* IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
* CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/

#include <fstream>
#include <stdexcept>
#include <utility>
#include <sstream>
#include "Resources.hpp"
#include "Encoding.hpp"
#include "Utility.hpp"
#include "Base64.hpp"
#include "Log.hpp"

namespace BearLibTerminal
{
	/*
	static const std::string kDefaultFont =
		"aaaaaa"
		"bbbbbb"
		"cccccc";
	//*/

	static const std::string kDefaultFont =
	"iVBORw0KGgoAAAANSUhEUgAAAQAAAAFACAYAAABTKqIKAAAAAXNSR0IArs4c6QAAAARnQU1BAACxjwv8YQUAAAAJcEhZcwAADsMA"
	"AA7DAcdvqGQAADJ0SURBVHhe7Z2JciM7rkTv///0TG/qpmEAJ8GliiVxIl68vgJJbIkEWCXb//3333//O/93YnAw8LEYmOP4//78"
	"7wWk6n8fAM7JQxRHysfT4//u/o3mJ4nPOPB+Ht4aWP3vUefO/jyHlI+nx+/d/RvND8TnEMBogHff/+4F8u7+jeILCeA1Htj/b8f5"
	"SP5zHY1gmXxU/+r9MxLg2ViJb2YD+U/5of27y9/dv9H4Q3z+Fe8IAYwUyaiDq/eP+NYGf1V8yX+yn/bvLn93/0bjD/EZJ4DIwNfo"
	"0StXO+RogGg/AYzkdD7JV51P+dkl/hSfp+Or17/R/PzJ/yEASgAVIMnpfJKvOv8QwO/nXxT/d5VPJQAL0lfQvPuH/Wz3AFMBknzU"
	"v1nnt+d4+em9ooz6N2O/gr8n++fFqDIBtGvNv+cwYKLgl+0kH3Vw5X4qQJKPAnzG+U+OvxK/d/dvFN+XEYBlpVe3j9jqCSMYFSDJ"
	"FQBnCZ5x/rsXyJkAviMoy3lTl2cCoAKlAiQ5nU/yGed/CgF40ybF9xPkt08AbWLarn8mACbgGQTw5PgrBfruBDd6BUjyP+ebgFmS"
	"Ipk1ioB+5H25evf4v7t/o7iH+PSBqjWKEkDyUQfPfv5ZAOqiT47hwddQ/scJ4MngObaf/H84Bg4APhwA5/dBfPbvxDgEcAjgYOCD"
	"MbBP8l93uQ9OxunGn92N78j//gTQPuQ55LBPvk4u3iIXuhNRh7afe+vazzJ5dlY2ISgkoUwYqo9PBj/5qMQ5+uZd79mKTvu0P7NB"
	"eUuVran6NxMPCk492zMbkthpBJAFf5QA1KK3JNJ++eP1nQIFSFGgKPAknwmClWcRQbdfzoqKJAMg4UHRXwV4ZLMXx6p9tH5mrnox"
	"VsG9WbueACx52G5NTtN6SwwvMohYfAUBKDaQnzOBVPXRs9+Sqs2DR8BeISoFlMVPiZuyhoit1x+7j/BH8l4cRDGA+I8TgGIwFchI"
	"AtvERsElG0f1KzaoOsjWEbkCkqjQ1S5LfnryEQLwGkyFACsNJotBFjfCRxYzz79eEnH0aASgJl8JvAJCZXSLGLunQAi0ma4oQVkH"
	"rRBVBQCZ7wrIKKYUJ0WH57tCAFGBQYf78qPoVDhETqN1UMF1D44VgjLn7kMAHcZ/+Y0ulmV7ioyC3gvwFcAhW628YnsPSRM5jI7Y"
	"o/sVn4hkqMtXc0KE62FayWOBaHQCGHVOHdUyRvcSoHaA3u7Y7quAqFKAvbFV91Fx2rgqfipx8YgvytcMfEREWymaylo1/so6yhGd"
	"0blfJ4BOBWGXrjC6TawyMlLAVAD3rKvotnGoTC6ebTRiK92hQgCEiypBVwtQOb9yZsV3Nf62k0f4oFiO7AvOvp8AvDGHQJp1kagL"
	"UFFS8ElO55N8dLyM9qt22w5MOfCIi2xQbKEJIbNrRvFmBJqRM+WP5FFDVHHTuU4nAFIwKq+Ao7dLZjaSfpKP+t9LXKT3CrvJhiP/"
	"+huIN4rHswhgdeB6O8gMu06h7oPFGfm0Z2ya3/cO+opEnjMPZt4IAyeZb5TMO36a7Oh89k8wHgI4BHAw8MEYOMn/4OSf7v3s7v0l"
	"f53PGA4BHAI4GHg6BpRXuIGP+u+lpyfk2XvOzEAy3srpPbFnp6I/ew9LNrbBjeyLzifmzvyJXh1SjDx7Z+WP3qNbuZpfNX4Ufxsz"
	"Wq/KFYxRLiMiUs7O8BsSHBW1PVRxQA2YZ7ByfvX1yog9WWFTsnoBq/jX+kQxG/V/9X4C7mz9swmgmi9lvYqt4clFIYDZCRg9rxrA"
	"WfqoUysFT8XqJdR2Zvrv2QAfjV9v3JR4ehMQ2Ut7ogmFiEqZIEcLW42lTAxRsLIOQwEmORXw6H46n+QKKWYBzuwfIYAXcKMCj3JG"
	"8Vwt9wouI7pqfir2v4q7kgc6X5kSqXBJrpKPXPivh58KaCgAbUeqBLbK8Lbz2f0kJ/a1fhAQ6bzIvuq5NkdZh/d8mJ0/ahq2g6rg"
	"VYlX9cfT20MAbbyzQu2VqfFRibREAhUCiAqkmhAqADqP9mdyjyRUfb0sHcVYTXyFACxYPdCo/lYJWgFoT5GQvRV5RAAZMSh5ImyQ"
	"XImduqabACKmqwS4B3DV80cIoMc+AkDFfg8IBA7aU5WTvRRf2t/jTwbuqr5V8Yj8In8rhUtnkbxU/D+vAcqBowmo7KfkeQ6SDxX9"
	"FSD2dEiFgKgA7RkUs6r/pL96Hq0fJdiocbV+tDaQPdVCJ/xtTQBe8F4O/fz/CmDb9RS8DKzqXkpgRhLWN8W/CsCoGDN9No5ZYWSA"
	"VnRExRF1kMi2WfGjIsn0E4EQYap46sUn+UaEW81VcQo43wIrBuyjvz6rdLsTz+81VYlbtrZyjpiHQwBioD668Bd3obeP7azCnXVO"
	"k89DAIcADgZWY0C5wmQ29O73rufmGnySvzr55/yDsY0xcJKzcXLefjQ+sb+9/r7+ssLXk/+TmNsTc4r/jX5Wf+N6+gf0BQ8Y3grE"
	"SnyUNRuD4a3ydeIsNbGvi3oA3LNHeeBRTeDogxLSp/ipfA8geu+rnE82PlX+Kb73vuJT9tnp/fXf0YPAP1iRWCLtDLOT13Nezx7l"
	"CyQzCkq1TV03w6bdzvgE38nHHnm7h/4dNEiNALLu6jFMtcvRGdTd1eC151gC8GQ/15BuWmNti2ylzyM7KHaqfZH/3v622yj+ZTZC"
	"h/orjgib9HtxjYql9bWqj3CiYjQi58iPdr2K7yZ/GgF4SqoBWrXeS9pMeyuTAoEts5UI4LXXnlEtgJXrMwD2Xvtm2vsC/kwCiJqd"
	"+rmKLyIQIo6AoK4nAApMb8KpQ/YSkJqgqLgzsGVE5eldAeDeeHsTlAdCBbjZmln2RQSqTkgKDhQSp0LtJUo6d4gAlBFOLbBeAshG"
	"rJUAUhKfraHRkM63BLLzBKCCkDAwc4Kj+FXiPzrBzShuhVC9+HUTwEwGVjoE6VPOmAkgAgh18U+ZADJwV2JE5ED4yOTeBDVjAqCi"
	"JHlGLBQ7irvXvF9x+LGXrwAjAfccGz3vyQQw2kF2vgIoWIr8H5ngqICvnphoClolp3O7JwAvwA2DuAQTsTjdkdrDsjGGugTp9+yP"
	"Oj0wKP51FpXwoquCF7NsqsjWK7I2NjYmXn6ou3nxy0h81EZqEGo+VPyp+FDjRIWsEm2Wu9IEUFW4+3pKxIj9K88esSvbW7W5un6V"
	"3XecO+I77R2Vd8aDrwCdB2/7tVIK9Ii/K88esesQwDjOZ+Q2O6NXNoiL8cAMGrCcKNQRtNePaHTvPe/qfVVgV9df7c9MfVfm9qa4"
	"vj8BzATEOevg5c0wcBL6Zgl1H0qqD6reORbHN7fWDwEcYOyBAe8Nw8nN8twsV7D8jj8KEnotNHr+E/ZfedeN4nHTHfh2fJLfysPB"
	"5rVe1Z95BKAaWjVWLdBe/XS+Mj7fWUCKfZW3AARI7yzq3qp8hCyvethL/lfwrcaFzuzJ2R8/1hOAWmCvwFbXR/vU8yrFQbp+ygeS"
	"UWXvUFfFBop3pShJL8krutq1VxW/V4ij8fNi0n42KoeYziGArAMqSa/sj86rfq6ArU24l2hLCMTois7Kmpd9nl7qGlkBZV2ud4RX"
	"cKDojQouw1AlplX/ZhMAnddDCIn/OgGoBUYOWGOU9UpyM4D1FmZbYG2xvzq91/F7gd4DUsW+nnOVQvQIhHT1xIbwYbFBOK0Qo+Ij"
	"2VeJSbW4La6VGjD2zCcAGs1sAiiA1QRn4K0C0CswSwQtCUREk/mo+hf51ZKRJSYCnyJX7COC7iVghWAV+2yOFL+pUSkTFGFbIZgW"
	"bxl5EbYDuU4A1dFINdwroGoHyhJaSYKX9NECq+qnRFrQRODu7XS9ee4pKmWPEj+PBJSzr1hTsT/LZUayrY4i2eoEQKNVVPAEaAqQ"
	"yvAqcMker8CiThR1/3ZyaAtR0a2saWM9SlBqEVTsUs9U1hE+LO6utlPBZ+ZDVLwRhiwWaT804nECyDr4zGTQWZk8Y8/K9FCZanoJ"
	"USkKsoNiVdFx91qVACgmd/lRLf6snpTi95pV9NmPzzUCUArIA13Ejm1XrNqgdnq7rjga/X0lpzA8+VAFMZ1npxTPxsoZCgkqGJil"
	"M/MvAfOvnO1EfpW8KzmsNDlx8lxLACsA0UsAu3aIK2P0RF3Vgq6u3xEXkQ8qCXvXgsDPeQRwN7goaDsm+u6YPUF/taCr671p4wlx"
	"UW2EKUQjgN1GK8/5ynikBu+s0/GxIlY9xTyyp+dqusLv2Wcmk8O8BPcEXnVUHX3U81asW+m/Yu/d+hUblWcNo+fsun/T/PwmANU4"
	"GrMVudVHD0pIPiPhK/1X7L9bP8WwJWAPLyvtb/W9OnThjlv++YrqdKlcIbIGpuBj4fX13wRASaTijoz0ktWOWkoArpgArvA/I9u7"
	"9UckQPlTwXknfojgFPnd+SH9ig/OmnlXgE4DpjD00X3yeDDQhYHxCWC0A8zeb8dVO0K204cniyaSaAqxn1f3k32r5TRdeZ2HfLTF"
	"qOqgKaFXTuTg5VCZaJXrEE24s+XZFcZi/8far88AegN8d4JH7lg0Wq2WzyZAZZSP7q1KsVvQzwbwivM8QnpqgVN8IvINrp/fHwCS"
	"gp3lCoDJ/iyAGbtWC+9qPUTuVBBt92jPong+Sd42EorHbnLPdiLrXxPAkxJE9ioEsEvh7WKHmv+o6NX9dxUMXaE8vyKyI/ztLnfq"
	"4z0JIOtQdD+19yR7vZgtv/MK4F2dsoKurt+hICi+Xn53IoAKgXXE+x8BeMn1lEcA2Wl/dMddMcJHozUB72kTgPXnVSTvMgHs6g/h"
	"aDD+7zkBZARAAbu6MCnBq+UUD7pWVffvuv7pBNA24AL+DwGMdvDR/asLnM6n69I7EIBH6nc9k6gSYCV/yoRr9L8nAdA1xWPIu64w"
	"dMdbLY/8zoBXie9q+5XzvcLwrjQRmasxWrXfu9sTMXhyx76ubw898tt71KkzkBzZ5+Dkw3L9WYk9JPBZ+f6wYu5pzJ8BiOgByQHI"
	"Z+T/5DnM8wHAAcfBwAdj4CT/g5PfMzKePeIv0n0Irr5+EYiMzkbplfdregpLdqtPgemcFf5Xricr9JPPJH+6/U/yrxJr8uuP/PxG"
	"oOwViw1iz2seeu+7g34CiwWe4pN3Zk/87Gs+G69eW8jnVq42tx7/KvardmS2f/seQM/7xIKCv9jxEumB33Ny5QRwpf9ZsVNyCVyR"
	"HwrA2jXZ+pEJZIb9nn7Fv0qxR2vvzM9M3IQEMCNI54znPlMhgD89t+/q36Bfz/+FIF73+vmZN6VQF7pDTtPNVfKoy1j9NrZ2sttR"
	"TteszEeaMHaRR9MZ2Pe9YxGj7CqPxthdEkRXjV3iunu8qvbR5FI9783Wfx4B0IO9qxNMxLBafrW/d+mLJimaIO+yt/eZTtFe/wpA"
	"I/ROcm98i6aB7Mk0AWSVfHWB0/lFwPwNw0UAHdanTgCU36f4W8zns38aUCn0XUZrFYizn0IfAshfddu7c7GAhgnqZn2fRwAUcLoi"
	"zJZTga6WUzyeLqf4Pd2/Qfvf6wpgn0jb5EfTQDT+XbE/usJEuqOE0wib+ajIskK6M37KW4hs+vLiH5G8nRayHLVnkI6qvHI2xOe5"
	"761ppK526up5Z/3nYGeXXHsNjK64Hlk0n31WEivB2iXpx47Pwmj1GZCK6WDdZwQ3Gt1OcX1G/p+aZ7pateO956Ow/wDgqeA4dh/s"
	"TsDAXkFUx5kJjp+fa9/s59op96P336xDVsfu7IFoBZvkM51F+0n+4/xDALvF4FPtIbAeApj/rd2tCIAA8KmF8Sl+U4G3cuFuK014"
	"hLlROeWOzh/dL5y/zwTw01hy+IlyFazROnX/E2MDr6j+um4fdhFZVGPhvYdv8Wj1ZfqFonNxPpJnss/z70+M9iGAatKesD4CirXd"
	"kl8GhnclypF8jhQPPUn35FGRjxZ/77OFjACALA8BjACvd6/tLtVzPBJoi6B63pPXU/ezRZXFLopDVUc1nur5PfYdAtjsQeero7TA"
	"9P5tx+Psv9su9WkTglo8GUFS567qOASwYdFVk7JyfTQBRMXrXRFW2veks9XizHxK7si/7uvR/7yrXA8Br/Ahu040+s4V4A6wr7gC"
	"3OHHDjrV4lQIoGfEtpPZXQTgPasQPjsEcAeIDwHMw93M7nkI4IzuU19FRg9gvNHR6yTq/vMM4N8rZLrPe0VOe1SSoXNGCYaaVaS/"
	"/dysmcfEZNynyu2ISiCIwEbgyh5yvXPs1eJUrgBRrhQdap6r+a/kLmoYHgH8mUKfQwBUAJVAvevaE6Pn4HkTDD4jYE8A9t023q1/"
	"E0BPvcJ9gE+HAGYl+VNH8FnxO+fcUou3KD0sfR62HgzsgYF/BHBGyEOGpwvXMUAPCEn+ijnV34g82XsI4IC+DvoTs7huqOCVV3Uz"
	"XlVKX1CK3g8S2xwAnKI5GJiLAaq5EXm4t0IAxFzRQzD7fpTO8b4k451hR6eV73Czd7zE+DTiZe9u2y/4RIyuxFexn+LX+5CT7FPl"
	"PXH04qdgPsqpivGs+1bxT34rcukKUPmiRMVBYq42SVQMStehgqzKKcB0Xo9/EUg9W0j/ajnlRMl/RHRUwMqYm/nfQwYKKVfqg+K3"
	"WP57jMnYfTaAog4fFZoKIHV/1Z8KASgElnUXLxdebioxqfpbXU8AVW1VO6PSnX+uySYmItIKAVfwQeTxspt89HyLajir7R/n8EPA"
	"KiBo/UoCGCnAaiIJRD2E5AHgCzv/AbZaVEoHpXyRnAgg6+6j+FM6eJZXZb+CqZEJhuI7Kof4zyMAYqAIKKqDwGR/xRHBeIVkO26m"
	"gzoUyRVCyAjAs9WeqYBVjXd0tu2uCgEoJJCRGuXOnq/EIdsTEYNKJoTBjPgoP9UGOo0AlCQoHafqgNLtFPAoBVhh8qsnAK8gleIj"
	"QFXlik7KsXcG5ZmKOitaj1gzgmtJjvRmpDA64Sg5p/xNI4DeAqomVnFICexse5UOr4BF8a+nA0WFSfqq8ioBUP5nFVBvgyL9Sk5H"
	"JpzR+NP+LgIgNs0YUjFo5QhE3YfsUwp9xH7SbxNGAKTzZst3I4BqAVfzq+TjSgKo5hPs//plhpZFqx3U21sJDDH4qDwDSnR2NCbu"
	"SACZD4pMzXcvAagxVicZIvqqP0QMr6aX1Ug27Sgyq2MG/uAaMffbTFVwnPUn/gcDczGgXrn+xH2u8pPME8+DgXsxcAhgjx+zPD/u"
	"evJwCwYOARzg3QK80/nv7fyd8ecvAnUefEB4iOhgoIAB6twkVx64O7V8COAQ3CM719uRCxU4ybsIIHrfryg7hXMK59MxQO/kvaKk"
	"1429r0FvJQB6x5sBxe6tko+3vwrMEfsVXVWflDO97yKs1lO162nrX+/hVbsfTwCqo0oBR1+8UHTMAm7POUoSFR8qMRo9zyv+mWd+"
	"6llVAniDOP37XQDZCGG//RRdHZSRh0YgLwlqh/bOpgmBCMD6bpNO+4kYaX9V7oEyi58ygSnxz+KU6SD9GdZ68JbF84U9WtNitIIP"
	"BfsW/+S/jUGxCX4lgNkBveK81QVGdysq0Cvty8gzsoPsJ/lO8bHFUsWfRwBZU6HcVvXPWE/5MA1iLgH0BIsApjBatkY9X+mSWXdV"
	"J6iM4XsAQAlX/Z9FEHdOSKsJQJkQqv7PzE9kX3JVmUsAPQCmALRnKh1udgLUAqPxblaBKfFqY0DrZ8ur8acuWrHPIwCLHxqxs5H6"
	"EEDz+wOvAjRNAFV5BVAEzgwsu8an6n91/Y4EoExnrzWHAP5kfBaAKww8Y4KoApAATkVO+0fl1fjN9r9qf1U/kWxF/w4TQMXeGXhX"
	"rj3pFYAA5o3gVqly98g69UtGY3SPnBKi+J9dQ+h8VR7FlApEsT+Lb9U+O0KP6l/hX2sj+efZT9cEikFEgj34rcQ3m3QCEhj/NhuN"
	"4EqCE4Yqf+WT7CH5TFuUs2bbc/d5s/UrMcwwphBAu7+i7w3WHgK4O4mzC+bu82brr+TH030IIK3x9yIABXzKmgroRtbOtmX2edWR"
	"coX+SnwPAZTrubyhPJJXEnjWnnwcDFyKgbXKlI5QGdEOONbm68T34+K71uFDAGvjewr2xHcQA2sD6L0uGTF49nkjtpy9a7Fz4ntJ"
	"fL//deCsyNRx/bVu9L2n9z51BTDIXtI5st/ujWKc6eiZtFqfZtj/emBYfXDYrn9hL8OZisEoZ9mDwh68jtqj+E/4U2Ie+MY/C2Df"
	"kZLDs+UWqEowKmvIXjpr9n6P9JTJh0iAwO0BkXy3wIsKODuntcvbT/Elvz2iyz6r6LP2er5QDO0exZ+MxCrE92OtTgAWIApbK+dT"
	"wD29UeDbThaxIq2pdgjF/grgegiAQENyr5AzcEf+UAGTvG02VXzZvHo2emsofyT3bK7ku5KbzP5W9opdVDuNnAlACUCk3AYncjbb"
	"77Gqx5oZKKOAzCCongRSVxqRK/nKSCYqJPr8FUsik0w+QgBRIRJ+lFgTvrNmkzVOoUD/qvY6u4frNg6Cb7XfCKR01daoqwmAOsyK"
	"DkAEoMqpwCryQgf4AjDqIkSYSvyzM0i/kr8qAVEHV+S9BEH1FBGELWzP54zkGxlPAApjR44IDPT3i0U0HYx2GAKX0kGoACKW9khR"
	"AVZPXFXCyQASTVhUgL0EIIL1F1aUCSda49nXE+PIXsKwJ6fPFH8HfKgHlJz0imDAwG8dqsrwFrQU0FF5BcwjcZlJNuSz14k8AtuV"
	"AFSyUwkma2wUS5JTw51cf+9NAF7Hqga4kjBi85lFO5tosokkmgi8WFYJuhqTSj6sT0RQCgGQ/qvlg5Ok/j2AqJgGDcArgNd97Ehd"
	"GVmzJBO7RvKsI2Td88oJgGyn/CoE0Z7R5sh+rtqikAORroeNCD89+SffSE74UOSEoyS3l3zbaNoPENnEeQBRPiPQKMCL9IyAe0Sv"
	"4veMNeQfgVGRj9qp2Diq4032fx4BEMv3FvasScjq3w3MGXnOmgAqxWW7227xqvhyw9rPIQAacQk4JFc6m3LGzgSgTk7RlUwlSbUQ"
	"lPFePetD1z2LAD40SdOuUCd+B+8GAycguxcFPUQaldPkYrt+zxQzu/NXcjbT/orenrVZbNWrazE/hwB6EnXlHu/BZ/vZqJyKc1YB"
	"FYE5beqZZf/qnCvxIRJQzrh1AugwcBoQVidw1fmjBU77V9l9zn1Ec73WyAUMhgRBpEPydkTueQimFEL2gJIKeIbcu0bYycCuyeSt"
	"z3RFofheIffszXyI1mf4iHJMmKrKaeIx8msJwCsGMlgpoGwNFXiP3I7g7RlZMUf+W0Bl//0qiLYA7bnWPpJ7JEJXAyoadT/ln+S9"
	"eiLMZPqqxajYFul75aSqk+L1RU6MFbG8ByjqYioDkk1Rx6PzR+UEGK/Ttd0r6ihECpG/OxKAF+MqwRKAKQ+98qhYPXKvTADeBOM1"
	"EMKKV4veHoqfkcc/CxAxECmwHYo6BZ0XEUJUANXzqusj8mv99kBj45kVhgUIdfBRuUcySvfqLRrCRBbj3gInIur1hQiCipT2R1jq"
	"rc+UABQGp4KJEkQMZztoBYAZoMjeqlwBZ5a0yE/rw8wOTwSR2WRlHllkMaTCU86PsFMhErKjQgCWoMkHJT6RfV5uMtIo4vnfBKAW"
	"ICnoJQBKgEdO3lhE9o3KewnAmxC8YrITVFRwMwkiskMpvEqH87ChFibZQueQnPBn9b9iFhWjRxKejqyYW53KeZ6PgPevBEDJVOSr"
	"CCA7V5kAIgKpEsIIAXiEFRXFzAJXJoAovkrhKB1OPZ/0UadU9SgF7xV9W/iKnGqmB3+VGKTne0VRNShiHZXtq/qoQ9J5VMDK/oyN"
	"M1B4BGC7L/03+d8j32kCoPhXwJ81hncjADUuX9YpXbFdQ0pG2bv3fGujkvioGL1iqJ7f+hERpLcm0k0dfFSe5diTeSQVkSJhwo65"
	"1fiv3B+REfk6Gh8VP9b3rOkGGObvASgJ7B3P1X0r1830T516svE+K2ZvH5EGyVfEdnVMV9j87mcGOWECUBhWDd6OwFhtkzrVeIXq"
	"xT7rSi3LZxNOpEvNI61bHVPSf+Tf63qIACojtdfBPGDukqTVYM3OX12IV8d45zxfHYuH6NMmgIc4gz8XsJsf70YAu8X32IP1jQse"
	"V1Szkx6N3LP1nPMOFm/AwAm6EvTgCerHk6MSu7Nm6xrb2rhTYD/+Es4poBODhRj4F9zK02rvPWj0AMh2T0+Pt0Z5ih29L6Un5fRQ"
	"MzrX2tQ+pc/OVPyL9itXkGxCofhntkU58GyNbFDsj+LofU7njchfuM5w1dqkPPTM8J75ncXYw6G1i3L0R85/GISeklcDbtlsZL9C"
	"AD3njxCABcVs/dXzVq+PwEggVQj+EED+l7sm4J//NNjOBBCx8YvJRwDkdQOPZStkkdlj9ZH+aoGtWj9CAAqAbcza3I7kN5rmPH1t"
	"LjK5bW7KhDhSX0r8oAHoBBCNO9UOM3MCUEbUir7Wx2wcVALfU3A0gs4GfM95HsizUTbDx0gce+JL/lL8Sb6CADL8KvGbRgAUcJUg"
	"KgUZdfjojIxNyT4KZkXe2lchSIV0KucR4JX4Uoe6YgLoJZiK/6+1vRPALHzZCSfLkW1YNDU6Ns7/fQAE0F4CoM7jdW+Fkb3EecXe"
	"JobIYAUBEAHPklN+KA9ZzHumgagwVX+J5GwuewlAsYcmzIzovPNVHEbrfpypXwEUB5XzCGA9BOIlrdfeDACUBMV/xb8kYX8x1Ouf"
	"or9CYJEdCvlGxRmRbU98K3jLJrCIiCieUTOhbq3kVyEAsO/ZBDAhAN8K6hDA1/fuBHCVAOgqEclJP8kPAfyOUJAnnQC8UZnGKxpp"
	"7EiUjUg0Pild0wMZfTYip71RfBRfvC7do48K08ux2t3tXm8fEQgVeIYhxU7a3yOn3GRTQVusNjaVWCgTxI81937LSmX9u+08+u/F"
	"STX+CqlVz8yuRSNn3bz3WYm9OVjna7k3N4yT/+n1Ov3AUySnSA4GnoOBQwCnqxwMfDAGTvI/OPmnUz+nU6/K1SEAIgB68kr7j/xg"
	"bGMMnORQcg4BHIwQRh4sryd39SuWHYPpfQdhRzuPTXU8f3jMtICdAtDi9OFgGr6nel+COTFdir15vxCk8k1Bm2jv21b0bar2m07K"
	"v8m+6Ise9O06kv88N/OP9itXkIygo2+ytTEjgp9xvtWX5bfFB/k/U/7S631T75VHyqclrGp+PcJT4t8ZX/4DAmpxRgbQleHKBLbJ"
	"I3s9MNjkZ+fRfiVhFXvJn1XyzI+q/VSA1fOq60m/lUdNJWomXqxm1gfZ78RDJ4BZACKGrBLCKCN7bJ91AJv0aG01Xh64qgC+Yr3X"
	"oTy9Vf+9DmvPoG466j8VEMmz7h2RAhFApV4U+4w+/wrQM4JFbEgOqgWfjUFeZ44C1/rmkYcFkaeXSCPrAJm/WQIj/9X49RRkVtgE"
	"zB590dQ0i2CIIKiA1Px4+VebisUnNTjCJ+zXfxqwJ6EKgKoAzlg2IiGlICPS886kZCr6vHgSAAnAV8ifNgGoBRRNIDbXUYNQ6qNS"
	"rGqcB+17PgEMBuBvTiojWkQAMwpwBGAz9PcScjTpVc5bQYA0qVTwM2pfDwEsjh9fAaIx2TJe1H0jYETMnHVc7yzbuVu7IhuzDk0F"
	"WJFH9kbTRjQVqOt79FlQKiNoNoUpE1pUlDRtZQSn+NGz3zvXawCEf5oao/1q/SkTiOP/0neM+F5Y7RrROHQ+H89fRtCz40u6PPLp"
	"sYH09JxJTWP0zJv2jwPoJsORXI5dWm5XF0vUmaJJ4eRNy9ukOF2q7BTthj99tpIAvDvvJOAeLM3B0iGAA8iDgQ/GwEn+Byf/VxeN"
	"/ndVXEg/yUftpPNJvoP+6AGjYNtXAAgbhkav7Ennat1POF95khw9Qa/cte0DLe/Mq+IVvXVo/Vlp37voJ+wE+Pj6GjB6MEOvoVSw"
	"XHHfHCmQlfYpMcr0q29Mqj5QAZDdo6RO+klO9pGczic5nU9yOp/kdL435TV7/l0BegGmAk5dpzikEFXUEXuKrNemNvh0hhIfypFy"
	"htJNPdBF9pNOIggCOMl7Jx91wiD9rX/ka4TbLCekHwo8+oMgr8/HngGMOk9F8XT5E+KjAIyK3/NTLUzST3K1ADIfRgrQEiARYpWA"
	"yf9B/fz7ACjA5LBSBHZNNMZ7DFvdm00PPR223ePtV+PT2lU5kwCgyKkAXiC0dtGEo+ae9JO8LRLK4ewCVOsjwqlS4IrNGeEmNTLv"
	"ZwGi8adaAARY6ka98gjMI8XogcMmPPLXKzp7nmczxc+TqwWWEUAGcA/ks0fwzgL4ZTb5rxSgSjw98Vf0R/4DHuYRgGpA1N1HCpc6"
	"kSJfTQBKfGiC8Iqsp+Btx6QCqE4AHmlmDYL0R3LvTPuZEp9e/RHBRyQf5XdEP00gUG/8HtiCTgmoBRh1gKh72KKJWFYp8Gyv0j28"
	"7qsAQI0XdRCvqHaaAFqgRbZG9vYWAMVEJYhV+lV89OpX8g8E8X0CGO3QBPjq+ep6KnCSVyeAqCu2nysJyoqFSKd6fm8BzpgAsgmo"
	"twBUArB+231P1W/t7sDDfgRAhUqd0guKMiHsQABKAjMwK/s9gs4KoKqvut6bDu0EGdl3CIB/pR800PlvAaoTAIwo7g2kdUq9otA1"
	"o1I80RUnGzmzDmiLQJ0K7JkeOSrxmUEAUXw9/TZ/T+3AdxOQkn+or7HvASgjSNSRd/ycpo8dbfbIsJIX6sCrfSb9JB+1j84n+d36"
	"CbMgPwSgFtBoolfv7wUCTQhX2V2ZILyi7LWT/Cd5r96oe0cTNMVHnTDN+fMIwBt/R4NzxX7lanCFHT06nmx7j79nj1+vyjUrwMo8"
	"AjjJObE8GHgcBu43mEasI/cjoI6Qn16UNJnOwlc0gm8e/z0IoPcp8OsJ59n/NY8z78ijAM6eTXjFSc8yvFxH/trPI30j+HlhMCKa"
	"6E4/GtdJ+w8B0FPed5BTl5sEpm8/eqoUf1tAXjFVH4pVfRnNL+nbngBaA+2/yblsDK3sHWHg0QS++/52Snr5mnXG0emBOjh1fdpf"
	"mQAUDM7IfxTPqD5au8hfVW5JXvH9x5qv3wTsIQDPQDLaBuAQQDzCjwKUgDCjQ0VNhHTvIB+N7wj+vcnMIwePNC25VEilWcsEQBOC"
	"EoDM0RkJOASSE0jb+e0UoBCA0l0ykO5Q6JENM/DXnmHjQPVBzVKVZ3Wa1N88AiADIoaakYBDAPMIgEbsDJBPJIEZ+LMEm01ECkFU"
	"urmniyayRs5/G7AyAUSdJnN6RgIOAVxHAFkn9fK/c/dvn49E11IFn4TvCkEQAUfyaEqDCeQ7cKzDVQKgYHjPHU4Br3sGQPlr80Hj"
	"agZOGlV3JQKlwAmf1vdKV07G8/DvNtCEUJ4A1AcK9mBbzN45SjAowEc+RhDRZKZ0GypsS/i7FvpVzwC8BqcQRFSD2d6o/jyCCEhh"
	"7gRABBDJT4GPFXgWvwiQXkEoE0C2xuumuxPCrAkg6soU02qB03plSmvOYALwRsTomhB1g2zMmZGAQyD5NwFpJMwmQMq/evauRDAD"
	"f1E9VCbkntrpmQAMgdS+CUjjYE+SZyTgEMB1XwWudqAeTFy5ZxR/yjUqmg6u8BMmkD0IgCaEI/8eAdu17YpV4FrRBFbZqpzrYasl"
	"BZJXivuO2IHO+wlASdJZU8vTynjdAeKV/rz72VMJ4N2Ddfzbh2hOLi7JxSVKhv6k+AHC2hzRiEvyk5+1+Vkc3+/GnxHv0Qktk+3s"
	"h2CLAVv279iT4vkQwKcD5BDAZxG+wTv/aTB6zTE6Io7sj95BV6eYkSfo8JoFO9bq/RSLUQIYfS0Y7ffsirDY5q+H0Efy376Lp1h7"
	"to3gv9Vt3wqJceC/DKQSgKjQ/UusSrKz4NlAVBJxJ4A9u72kRrH1gBsBKorJrgSg4ClqAMreqGAq2JlQgN9+i1LF9gn6mQCIoShg"
	"V+2PWLwFuPdvhQCyDkEgpA5v97+S6gH0Zb+nM+qC5N9qAlD027wo/239zQguio1Ktmr+o/Oy/dS5q/VTjLdOANSFRuUqgJUxql0T"
	"FZgHfIVRswBTsdtxMZpcogI4BJD//goPQ17MInyM5p+m1IxwIgJTCUKdaIyeMQIgdlIYtshY7p2aGNazM7raRMRAAaYOpHYHqz/q"
	"9taeEf1eLNrzSZ7FV81vlEOVcL04tdNUNT5EHFSYqlxdlxHEwIQyRgBU4D0EESWcJoQKCUTTisLQSoF6HaYCQG8CIP9VgsmK2St6"
	"jxA9+1SCVAkhIris0L2itZ9VCKVnIogaS4Y5qqMROcR7LQH0GN4DEKsnKlKviEgfJZT2R4D1CiYCsAdiKrjofC821OFJ3kMIPfZ7"
	"eVZitmrfDIKYMQEojSvQ82wCGO0GPQmsFHz1/FEwV/W91lOBk/wQwP/Cp/mEl9UE0GLCmUI/gwDsaEvXjKjre2M8fTZTTgShdH3P"
	"dypwknsE4NlqwdhLWBQH2xiyCcArkMoECQXm/lovG08vLhkGs/2eLJkQ9vsWlFcwrVMkz8ahjG3pXJJXbKycFSV01ucW7HYqIPlE"
	"MOKXplSfKb6RvHdfL+aiIi8WsRS3wLfPJIDVoB0BkgryWetettr/b8kykq+OZY+fI/Fv/ax06hGdWWOqNJYsVlsTAIGL5D0g8QBe"
	"SfiIzrN3TePpLd6RfNyhs8ferQmgx6GzZ00Rnbh+VFw/ylnprnR1AYyO4KP7yV/voaH3XIDOOfJ7ay3I2b1GHVD8fgvjPQxqrygr"
	"5UoO6ArWkgTdhRV9Z838ujwE8OMvIe8IrCcQAMXtEMCe2LIPEJ08fv8eQJXBR0ZEr7Ncqf9nQDx9FRtorSJf2eGJYNoYZLmkIo8e"
	"ho3G17OPYpo9Oa/s9fBx9/4IswpJb0cA7xBgAoQiv5sAFCIkAogAGJFK5TnCjCbTXqmu0r0C35cTgBJ8C44KAytdJwMf2VeV2wBT"
	"B7NTTAb46KxdCUDpvm0xef8m4qD4KkWU5aCa/13Xv2JrJy0lfq8Ydk0AFBBiVtq/k9wGudoZvVgo/u1MANkVwSPLiEAtcCPceOuU"
	"GFry8XJZIROy7w55hs+IIFo7XQKoJDgbPzwmskFS2Gok2aMJJgIg+ScSQDQlVAskIogZOfUmhBH7IrIZtZX2E/4i0ksngNEO3h6u"
	"kIDX7UYCOgrAaH8U7KjDtX61cVAIbecJwCueNl8eyUex6CFIKgqSK/FfSRCj+LRYyvDXRQCUYDWAGZFYHRlAqvqIUCgB1pYMUJ4f"
	"NNUo/uxOAB7ovBx6XbwaHyroqlyJ/0gDovNH8TkjfkMTgFdAMwuYAkhyJcDUkVry8gCWyasJ8vzZlQCiSSb7fHRCWr2f8DRbPorP"
	"Kr4i/HpT24+1/77A4ClS5FHAonHEdoloArH7szEt6kY2+JbMPPknEkA0oakEYAmy7TijAJ69f1aBq/gexeao/15uGpv4G0xRgUbF"
	"kxVQa0zASF9+gUK1QGedn/k2W+bFwY6kKyeE2f4o5/U2m6dja8TvRdhmAlAVK9OCetZooEb3KyCetYYIdrV8lh/qOSO5Gdk7A3uj"
	"+u/e7+RoDQGoYDjr9Pg/OVbRFcMWJRVIbwzu1t9r9wX7PgOAFwRyyx80On4ffAMG7v1hoBkApYc6M3RkZ4zqv3v/6vjQ+fSQq7p/"
	"1RQR2fHw/P1myGxEIgdJTgmcIR+xf1T/qP9375/lv/esQjn7EMDvCKjPKGxM6RkRTgCUpHcBaG+AKT4kH41fS9D236TbW391h1Rs"
	"rExY1RjMiP+ID6P6R/fLBJABgx6iZElRHRjtIMoUEPmo+KeAVOlm7Rrl3woBKL5HBDjYQf5ub+Pj+RXll2JG8tEJlvYrxT8Sf9JP"
	"9aPKAxv5CjAagNX7RzsaBZDsH9mvEoBCPrMmnGo8lQKt2H91hx/J34wJ62b92kPAqxiOwNTaUQH8VfZH4FX0R2t+ft76av+tAEid"
	"cLL4Z/ZZGyNi83IW2f86U8VEhgcl/u+6H/ChEUA03vUwYGZQ1n16u6VSIJl/tntRh+yRqwDNioJGfCJMj1y9zq3kLyMAq8db27tf"
	"wSPh4Wo54WuxPXsSQIX1bfG03UgBBCXgCjl1aPLR3iOrI7dKgMqEYGOuFrhC8F6cZhVIlAP1/N793jMAi+GRBixPAEoX8rqEB86o"
	"CNX9nsNREXigVPbbjkgJvEvu2dlT4F7slQ7fMzmoRR8RV8/+3glodYEr9UEETA1Aqd0gPjwBqAGakYBMV1ToKgFERXOlf9FE4nVN"
	"IjE13jMmmArAogmgbQpRt/d8ypqGMuHtkN/Id8X+Sv68WpAnABVQKwP6aQQQFUL2eTZiK4Cq5o8ASKC7Wl717+nr1QmtewKgOwoF"
	"sLLf6zS2A2T/7XULT38EapoyZstVAmiTF/078n2G/9mIenWBZyQ3Oz+2aHY8P8u7YP/5YYnoanA+97FBV64Tt31rysndvsYeIO2R"
	"G2XCO7naI1dZHgLi3t/wA677c+RdzU5e7s8L5UDI2/5OkJNHfnJ4MNCNAX4NqAY3eqBF+0dHTMtyykOR7Ml29Y47qj/aT3GLHvBU"
	"7Vf1nHXdRbbzL4uZQwBU/MqTe3qdEQHw59mj4CT76W41ot+zv+LT6P4R28/ex5MCOzDaobPXNgqASH+lWBR93pqMIEb1jxbw6P7e"
	"mJx9XDsPiBFPAEoBtiC8Yz0F2huzs4kj88GT9eh/xSwq4IhY7OcZAWTkNEpc5PORP4IgmAAokVTwV+xXdGRXCK+o1AJR1lULMSIH"
	"tdtbQvaeeVDMjvwRBTx6/WUCoALfQU5gpQLMCMD6115pvMlCLdJ2AokKtNLt2zOyfQphUTyP/G3I4RBAVuDe8wv7mVJQREBZh86u"
	"Ci9blA7v+XkK+W0KuXcSOARgu7hX4DQhZIXkTQlZt/aKepRAyIZDBB9LBPw7AWnE90ZiC3B7J40KqgVq9JDOs4cATAXUdllrg0IQ"
	"o/rPBPCxBdjbuWft+xp4GoejkTgqVm89jaLqnTu7QysjsWezUuweQawigPMM4BADYWtQ/i/AUWEqE0BGDOr+jERooqAgzJgAokln"
	"5TOAGQTg2afYTDE98rcgp69/FShKqtcZbcEqRUbdnyaMnomCzuzt+vbaoDwH8NZEBRrFU7kCtbZlOT1F/BZFPHIdqAVAKeDeQhgB"
	"o9LRlDW9NihnZwVdubK8CC27Ain29Pp69tVqZvN46c7sWvy2IO7oeErBUUfPpqyMIOy+0TxtDtiRbnf2/vg7oCa/OQFUQBmNt/ah"
	"2QqAKQW4Qm/WhatdvWLf3f5WbD1r9SZ7daz+DyRye+/convXAAAAAElFTkSuQmCC";

	typedef std::pair<std::wstring, const std::string&> BuiltinBinaryResourceInfo;

	static BuiltinBinaryResourceInfo kBuiltinBinaryResources[] =
	{
		BuiltinBinaryResourceInfo(L"DefaultFont", kDefaultFont)
	};

	std::shared_ptr<std::istream> Resources::OpenStream(const std::wstring name)
	{
		LOG(Trace, L"Reading stream \"" << name << L"\"");

		if ( name.empty() )
		{
			throw std::invalid_argument("empty resource name");
		}

		if ( name[0] == wchar_t(':') )
		{
			if ( name.length() == 1 )
			{
				throw std::invalid_argument("invalid resource name");
			}

			// Internal binary resource
			std::wstring demangled_name = name.substr(1); // Drop leading colon
			for ( auto& i: kBuiltinBinaryResources )
			{
				if ( i.first == demangled_name )
				{
					auto result = std::make_shared<std::stringstream>();
					auto blob = Base64::Decode(i.second);
					result->write((char*)blob.data(), blob.size());
					return result;
				}
			}

			throw std::runtime_error(_FMT("resource \"" << Encoding::UTF8.FromUCS2(name) << "\" is not found")); // FIXME: remove _FMT
		}
		else
		{
			// Regular file
#ifdef _MSC_VER
			auto result = std::make_shared<std::ifstream>(name, std::ios_base::in|std::ios_base::binary);

			if ( !result || result->fail() )
			{
				throw std::runtime_error(_FMT("file \"" << Encoding::UTF8.FromUCS2(name) << "\" cannot be opened"));
			}

			return result;
#else
			std::string name_u8 = Encoding(L"utf8").FromUCS2(name);
			return std::shared_ptr<std::istream>(new std::ifstream(name_u8, std::ios_base::in|std::ios_base::binary));
#endif
		}
	}
}
