/*
* BearLibTerminal
* Copyright (C) 2013 Cfyz
*
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
* of the Software, and to permit persons to whom the Software is furnished to do
* so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
* FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
* COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
* IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
* CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/

#ifndef SPRITECONTAINER_HPP
#define SPRITECONTAINER_HPP

#include <cstdint>
#include <string>
#include <map>
#include <memory>
#include "Size.hpp"
#include "Texture.hpp"

namespace BearLibTerminal
{
	class SpriteContainer
	{
	public:
		SpriteContainer();
		~SpriteContainer();
		void Clear();
		bool Add(std::uint16_t code, const std::wstring& filename);
		void Remove(std::uint16_t code);
		bool HasSprite(std::uint16_t code);
		void Draw(std::uint16_t code, Point location, Size cell_size); // location is in cells
	private:
		struct Sprite
		{
			std::unique_ptr<Texture> texture;
			Size bitmap_size;
			Size texture_size;
		};
		std::uint8_t m_flags[0x10000]; // boolean flag for quickier lookup
		std::map<std::uint16_t, Sprite> m_sprites;
	};
}

#endif // SPRITECONTAINER_HPP
