/*
* BearLibTerminal
* Copyright (C) 2013 Cfyz
*
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
* of the Software, and to permit persons to whom the Software is furnished to do
* so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
* FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
* COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
* IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
* CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/

#ifndef BEARLIBTERMINAL_TERMINAL_HPP
#define BEARLIBTERMINAL_TERMINAL_HPP

#include <mutex>
#include <memory>
#include <deque>
#include <condition_variable>
#include <vector>
#include "Window.hpp"
#include "Color.hpp"
#include "Options.hpp"
#include "Encoding.hpp"
#include "Scene.hpp"

namespace BearLibTerminal
{
	class Terminal
	{
	public:
		Terminal(int codepage);
		~Terminal();
		int SetOptions(const char* options);
		void Refresh();
		void Clear();
		void Clear(int x, int y, int w, int h);
		void SetColor(Color color);
		void SetBkColor(Color color);
		void SetBlending(int mode);
		void Put(int x, int y, int code);
		int Print(int x, int y, const char* s);
		int HasInput();
		int GetState(int code);
		int Get();
		int GetStr(int x, int y, char* buffer, int max);
		int UseImage(int code, const char* filename);
	private:
		void PutUnlocked(int x, int y, int code);
		void Render();
		bool AddGlyph(const Bitmap& bitmap, wchar_t code);
		bool PrepareGlyphs(const std::list<wchar_t>& codes);
		void ConfigureViewport();
		void RenderBackgrounds();
		void RenderGlyphs();
		void OnWindowClose();
		void OnWindowRedraw();
		void OnWindowInput(Keystroke keystroke); // FIXME: const&
		void OnWindowActivate();
		void ConsumeStroke(const Keystroke& stroke);
		void ConsumeIrrelevantInput();
		Keystroke GetStroke();
	private:
		std::mutex m_lock;
		std::unique_ptr<Window> m_window;
		std::deque<Keystroke> m_input_queue;
		std::condition_variable m_input_condvar;
		std::vector<std::int32_t> m_state;
		Options m_options;
		volatile enum Status { kUninitialized, kNormal, kClosed } m_status;
		volatile bool m_flag_custom_render;
		volatile bool m_flag_enforced_redraw;
		Scene m_scene;
		Encoding m_encoding;
	};
}

#endif // BEARLIBTERMINAL_TERMINAL_HPP
