/*
* BearLibTerminal
* Copyright (C) 2013 Cfyz
*
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
* of the Software, and to permit persons to whom the Software is furnished to do
* so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
* FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
* COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
* IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
* CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/

#ifndef BEARLIBTERMINAL_TILESET_HPP
#define BEARLIBTERMINAL_TILESET_HPP

#include <cstdint>
#include <vector>
#include <list>
#include "Color.hpp"
#include "Texture.hpp"
#include "Bitmap.hpp"
#include "Size.hpp"
#include "Rectangle.hpp"
#include "Point.hpp"

namespace BearLibTerminal
{
	class Tileset
	{
	public:
		Tileset();
		size_t GetTileCount() const;
		Size GetTileSize() const;
		void Clear();
		void Dispose();
		size_t Add(const Bitmap& bitmap);
		size_t Add(const Bitmap& bitmap, const Rectangle& area);
		void Bind();
		void Draw(size_t index, Point location) const;
	protected:
		Texture m_texture;
		Size m_tile_size;
		std::vector<Point> m_coords;
		std::list<Point> m_free_slots;
		Bitmap m_canvas;
		bool m_canvas_is_dirty;
		SizeF m_relative_tile_size;
	};
}

#endif // BEARLIBTERMINAL_TILESET_HPP
