/*
* BearLibTerminal
* Copyright (C) 2013 Cfyz
*
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
* of the Software, and to permit persons to whom the Software is furnished to do
* so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
* FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
* COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
* IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
* CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/

#ifndef BEARLIBTERMINAL_UTILITY_HPP
#define BEARLIBTERMINAL_UTILITY_HPP

#include <istream>
#include <ostream>
#include <sstream>
#include <list>
#include <map>
#include <algorithm>
#include <functional>
#include <cmath>
#include <mutex>
#include "Point.hpp"
#include "Size.hpp"
#include "Rectangle.hpp"

#ifdef _WIN32
#define _subject std::string(__FUNCTION__)
#else
#define _subject (demangle_typename(*this) + "::" + std::string(__func__))
#endif

// FIXME: remove
#define _FMT(x) ([&]() -> std::string { std::ostringstream _fmt_stream; _fmt_stream << x; return _fmt_stream.str(); }())

namespace BearLibTerminal
{
	// FIXME: remove
	class StringFormatter
	{
	public:
		template<typename T> StringFormatter& operator<< (const T& value)
		{
			m_stream << value;
			return *this;
		}

		operator std::string()
		{
			return m_stream.str();
		}

	private:
		std::ostringstream m_stream;
	};

	// FIXME: remove
#define _AFMT StringFormatter()

	template<typename char_t> std::basic_string<char_t> trim(const std::basic_string<char_t>& what)
	{
		if ( what.empty() ) return what;

		const char_t whitespace[3] = { char_t(' '), char_t('\t'), 0 };
		size_t left = what.find_first_not_of(whitespace);
		size_t right = what.find_last_not_of(whitespace);

		if ( left == std::basic_string<char_t>::npos )
		{
			return std::basic_string<char_t>();
		}
		else
		{
			return what.substr(left, right-left+1);
		}
	}

	template<typename char_t, typename func> void split(const std::basic_string<char_t>& what, char_t on, func f)
	{
		size_t left = 0, right = 0;

		auto insert_fn = [&what, &f](size_t left, size_t right) -> void
		{
			if ( left < right ) f(trim(what.substr(left, right-left)));
		};
	
		for ( ; right<what.length(); right++ )
		{
			if ( what[right] == on )
			{
				insert_fn(left, right);			
				left = right+1;
			}
		}

		insert_fn(left, right);
	}

	template<typename char_t> std::list<std::basic_string<char_t>> split(const std::basic_string<char_t>& what, char_t on)
	{
		std::list<std::basic_string<char_t>> result;
		split(what, on, [&result](const std::basic_string<char_t>& s) { result.emplace_back(s); });
		return result;
	}

	template<typename char_t> std::basic_string<char_t> strip(const std::basic_string<char_t>& what, const std::basic_string<char_t> of_what)
	{
		std::basic_string<char_t> result;

		for ( size_t i=0; i<what.length(); i++ )
		{
			if ( of_what.find(what[i]) == std::basic_string<char_t>::npos )
			{
				result.push_back(what[i]);
			}
		}

		return result;
	}

	template<typename char_t> std::map<std::basic_string<char_t>, std::basic_string<char_t>> parse_dictionary(const std::basic_string<char_t>& dictionary)
	{
		std::map<std::basic_string<char_t>, std::basic_string<char_t>> result;
		std::list<std::basic_string<char_t>> pairs = split(dictionary, char_t(';'));

		for ( auto i: pairs )
		{
			size_t eq_pos = i.find_first_of(char_t('='));
			if ( eq_pos == std::basic_string<char_t>::npos )
			{
				result[std::basic_string<char_t>(1, char_t('@'))] = i;
			}
			else if ( eq_pos != i.length()-1 )
			{
				result[trim(i.substr(0, eq_pos))] = trim(i.substr(eq_pos+1));
			}
		}

		return result;
	}

	template<typename char_t> std::basic_string<char_t> tolower(const std::basic_string<char_t>& s)
	{
		std::basic_string<char_t> result = s;
		std::transform(result.begin(), result.end(), result.begin(), ::tolower);
		return result;
	}

	template<typename char_t> std::basic_string<char_t> toupper(const std::basic_string<char_t>& s)
	{
		std::basic_string<char_t> result = s;
		std::transform(result.begin(), result.end(), result.begin(), ::toupper);
		return result;
	}

	template<typename T, typename char_t> bool try_parse(const std::basic_string<char_t>& s, T& out)
	{
		std::basic_stringstream<char_t> stream(s);
		stream >> out;
		return !(stream.fail() || stream.bad());
	}

	template<typename T, typename char_t> T parse(const::std::basic_string<char_t>& s)
	{
		T result;
		try_parse(s, result);
		return result;
	}

	template<> inline bool parse<bool, char>(const std::basic_string<char>& s)
	{
		if ( s == "1" || s == "true" ) return true;
		return false;
	}

	template<> inline bool parse<bool, wchar_t>(const std::basic_string<wchar_t>& s)
	{
		if ( s == L"1" || s == L"true" ) return true;
		return false;
	}

	template<typename T> std::string to_string(const T& value)
	{
		std::stringstream stream;
		stream << value;
		return stream.str();
	}

	template<typename T> std::wstring to_wstring(const T& value)
	{
		std::wostringstream stream;
		stream << value;
		return stream.str();
	}

	template<typename T, typename char_t> T extract(std::basic_istream<char_t>& stream)
	{
		T temporary;
		stream >> temporary;
		return temporary;
	}

	template<typename char_t> bool ends_with(const std::basic_string<char_t>& what, const std::basic_string<char_t>& with)
	{
		return !what.empty() && (what.find(with) == what.length()-with.length());
	}

	// ------------------------------------------------------------------------

	template<typename T> inline bool is_power_of_two(const T& value)
	{
		return (value != 0) && !(value & (value-1));
	}

	template<typename T, typename func> void for_each_point(const BasicRectangle<T>& area, func f)
	{
		for ( T x = area.left; x < (area.left+area.width); x++ )
		{
			for ( T y = area.top; y < (area.top+area.height); y++ )
			{
				f(BasicPoint<T>(x, y));
			}
		}
	}

	template<typename T, typename L> T GetLocked(const T& reference, L& lock)
	{
		std::lock_guard<L> guard(lock);
		return reference;
	}

	template<typename T, typename L> void SetLocked(T& reference, const T& value, L& lock)
	{
		std::lock_guard<L> guard(lock);
		reference = value;
	}

	struct GenericGuard
	{
	public:
		inline GenericGuard(std::function<void()> f): m_func() { }
		inline ~GenericGuard() { if ( m_func ) m_func(); }
	private:
		std::function<void()> m_func;
	};

	template<typename T> struct ScopedSet
	{
	public:
		template<typename U> ScopedSet(T& ref, const U& set_value, const U& reset_value):
			m_ref(ref), m_reset_value(reset_value)
		{
			m_ref = set_value;
		}

		template<typename U> ScopedSet(T& ref, const U& set_value):
			m_ref(ref), m_reset_value(ref)
		{
			m_ref = set_value;
		}

		inline ~ScopedSet()
		{
			m_ref = m_reset_value;
		}
	private:
		T& m_ref;
		T m_reset_value;
	};

	/// Round up to next higher power of 2 (return x if it's already a power of 2).
	inline int pow2roundup(int x)
	{
		if (x < 0)
			return 0;
		--x;
		x |= x >> 1;
		x |= x >> 2;
		x |= x >> 4;
		x |= x >> 8;
		x |= x >> 16;
		return x+1;
	}
}

#endif // BEARLIBTERMINAL_UTILITY_HPP
